package App::zipdetails;

our $VERSION = 2.108 ;

=head1 NAME

App::zipdetails - display the internal structure of zip files


=head1 DESCRIPTION

This is a stub module, see F<bin/zipdetails> for details of the app.

=head1 AUTHOR

Paul Marquess F<pmqs@cpan.org>.

=head1 COPYRIGHT

Copyright (c) 2011-2022 Paul Marquess. All rights reserved.

This program is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.

=cut

1;